/*
 *
 * (c) Copyright 1992 by William R. Pringle - all rights reserved
 *
 *				**** BLD DISC ****
 *
 * This routine builds a discography
 *
 */

#include <stdio.h>
#include <string.h>
#include <ctype.h>

bld_disc(Prog,mfp,dfp,artname,mastdir,chkflg)
char	*Prog;		/* program name */
FILE	*mfp;		/* database master file */
FILE	*dfp;		/* output discography file */
char	*artname;	/* artist name to search for */
char	*mastdir;	/* name of master file */
int		chkflg;		/* flag for checking w/ user on matches */
{		/* read all files in database & produce discography */
	char	fname[BUFSIZ];		/* buffer for file name */
	char	buf[BUFSIZ];		/* input buffer */
	char	album[BUFSIZ];		/* album line */
	char	cut[BUFSIZ];		/* cut line */
	char	*p, *q;				/* used for scanning */
	FILE	*fp;				/* albums data file */
	int		albflg=0;			/* album line written flag */
	int		NotEof;				/* not EOF flag */

	rewind(mfp);		/* rewind master file */

	while (fgets(buf,sizeof(buf),mfp) != NULL)
	{		/* for each data file */
		for (p = buf; *p != '\0' && *p != '|'; p++);	/* end name */
		*p = '\0';		/* just use name */
		if ((fp = fopen(buf,"r")) == NULL)
		{		/* file not there - try same directory as master file */
			strcpy(fname,mastdir);		/* start with master file name */
			strcat(fname,buf);
			if ((fp = fopen(fname,"r")) == NULL)
			{		/* bad data file */
				fprintf(stderr,"%s: Can't open data file %s!\n", Prog, buf);
				continue;
			}
		}

		*buf = '\0';		/* clear buffer */
		NotEof = 1;			/* not EOF on data file yet */

		while (NotEof)
		{		/* for each line in data file */
			strcpy(album,buf);	/* save, just in case */
			   /* extract artist name(s) */
			for (p=buf; *p != '\0' && *p != '|'; p++);
			if (chk_art(buf,artname,album,NULL,chkflg))
			{		/* artist matches - output entire album */
				fputs(album,dfp);	/* write album line */
				while (NotEof)
				{		/* keep reading cut lines */
					if (fgets(buf,sizeof(buf),fp) == NULL)
					{
						NotEof = 0;
					} else
					{
						if (*buf == '\t' || *buf == ' ')
						{
							fputs(buf,dfp);		/* write cut line */
						}
						else break;	/* found an album line */
					}
				}
			} else
			{		/* artist doesn't match - check for guest artist */
				albflg = 1;		/* signal album not written yet */
				while (1)
				{		/* check cut lines */
					if (fgets(buf,sizeof(buf),fp) == NULL)
					{		/* EOF on data file */
						NotEof = 0;		/* signal EOF */
						break;
					}
					if (*buf != '\t' && *buf != ' ') break;	/* not cut */
					strcpy(cut,buf);	/* copy cut line, just in case */
					/* see if a guest artist field is present */
					for (p=buf; *p != '\0' && *p != '|'; p++);	/* title */
					if (*p == '|') p++;
					while (*p != '\0' && *p != '|') p++;	/* time */
					if (*p == '|') p++;		/* skip past time */
					if (*p != '\0' && chk_art(p,artname,cut,album,chkflg))
					{		/* guest artist matches - print title & cut */
						if (albflg)
						{		/* first time for album */
							fputs(album,dfp);	/* output album line */
							albflg = 0;
						}
						fputs(cut,dfp);		/* output cut line */
					}
				}
			}
		}

		/* done with data file - close it and do another */
		fclose(fp);
	}
}

chk_art(buf,artname,line,xtra,chkflg)
char	*buf;		/* input buffer */
char	*artname;	/* artist name to check for */
char	*line;		/* actual input line */
char	*xtra;		/* optional extra (album) line */
int		chkflg;		/* flag for checking w/ user */
{		/* see if artist matches */
	static	int		chkart=1;	/* whether to ask or not */
	static	char	oldart[BUFSIZ] = "";	/* user to accept */
	char	ans[BUFSIZ];	/* input buffer */
	char	*p, *q;			/* scanning pointers */
	int		len;			/* length of match pattern */

	if (strcmp(artname,oldart))
	{	/* new artist, ask */
		chkart = 1;	/* signal we should ask user */
	}

	len = strlen(artname);	/* length to compare */

	for (p = q = buf; *p != '\0' & *p != '|'; p = q)
	{		/* check for another artist */
			/* find end of field */
		while (*q != '\0' && *q != '~' && *q != '\r' && *q != '\n') q++;
		if (*q != '\0') *q++ = '\0';		/* terminate sub-field */
		while (*p != '\0')
		{
			if (strncmpi(artname,p,len) == 0)
			{		/* name matches so far - ask user */
				if (chkflg == 0 || chkart == 0) return 2;	/* don't ask, just accept it */
				printf("Possible match found:\n");
				if (xtra != NULL) fputs(xtra,stdout);
				fputs(line,stdout);
				printf("Accept (y, n, or Y to accept all): ");
				gets(ans);		/* get answer */
				if (*ans == 'y') return 1;
				else if (*ans == 'Y')
				{		/* accept all matches for this artist */
					chkart = 0;		/* signal don't ask */
					strcpy(oldart,artname);	/* rememember who not to ask about */
					return 2;
				}
			}
			p++;		/* next character */
		}
	}
	return 0;
}

#ifdef NOCASE
strncmpi(str1,str2,num)
char	*str1;
char	*str2;
int		num;
{		/* do case insensitive compare of 'num' characters in str1 & str2 */
	int		i;		/* loop variable */
	int		c1, c2;	/* characters */

	for (i=0; i < num; i++)
	{		/* compare characters */
		c1 = isupper(str1[i]) ? tolower(str1[i]) : str1[i];
		c2 = isupper(str2[i]) ? tolower(str2[i]) : str2[i];
		if (c1 != c2) break;
		if (c1 == 0 || c2 == 0) break;
	}
	return (c1 - c2);
}
#endif
