/*
 *
 * (c) Copyright 1990 by William R. Pringle
 *
 *				**** CD.H ****
 *
 * EMACS_MODES: c, !fill, tabstop=4, !lnumb
 *
 * This file contains definitions for the CD program
 *
 */

#include <errno.h>
#include <time.h>

#ifdef __MSDOS__
#include <stdlib.h>
extern int _fmode;
#define	FMODE_WRITE	"wb"
#define	FMODE_APP	"ab"
#define	FMODE_READ	"rb"
#else
#define	FMODE_WRITE	"w"
#define	FMODE_APP	"a"
#define	FMODE_READ	"r"
#define	SEEK_SET	0
#endif

#ifndef EOS
#define	EOS	'\0'
#endif

#ifndef EXTERN
#define	EXTERN	extern
#endif

#define	MAXNDX	100			/* max data files */
#define	MAXOPEN	5			/* max data files opened at one time */
#define	MAXKEEP 30			/* max. albums to be kept */
#define	MAXMATCH	100		/* max. albums to match */
#define	MAXCUTS	500			/* max. cuts in albums being kept */
#define	MAXTAPE 50			/* max. entries on a tape */
#define	MAXSIDES 4			/* max. sides of tape(s) */

#define	MASTFLDS	2		/* number of fields in master records */
#define	DATFLDS	5			/* number of fields in data (album) records */
#define	CUTFLDS 3			/* max. fields in cut records */
#define	KEYFLDS	5			/* no. fields in key records */
#define	ARTFLDS	5			/* no. fields in album records */
#define	SPLITFLDS	5		/* max. subfields */

#define	KEYLEN	12			/* length of key field */
#define	KEYFMT	"%-12.12s|%.3d|%.3d|%ld|%ld\n"

	/* field positions for data record */
#define	DAT_ART		0		/* artist field */
#define	DAT_TIT		1		/* title field */
#define	DAT_DATE	2		/* date of recording field */
#define	DAT_LBL		3		/* label name */
#define	DAT_CAT		4		/* catalogue number */

	/* field positions for cut record */
#define	CUT_NAME	0		/* name of song */
#define	CUT_TIME	1		/* time of cut */
#define	CUT_ART		2		/* guest artist (optional) */

	/* field positions for key record */
#define KEY_KEY		0		/* search key */
#define	KEY_FILE	1		/* file number */
#define	KEY_REC		2		/* record number within file */
#define	KEY_LOC		3		/* seek location of album record */
#define	KEY_CUT		4		/* seek location of cut or album (see below) */
	/* NOTE -
	 * For title key file, KEY_CUT & KEY_LOC will always be the same
	 * For album key file:
	 *	if KEY_LOC == KEY_CUT, it is an album record
	 *	if KEY_LOC != KEY_CUT, it is a cut record
	 * For cut key file, all fields are used.
	 */
#define	FTYPE_BAD	0
#define FTYPE_CD	1
#define	FTYPE_CASS	2
#define	FTYPE_REC	4

char	*get_key();
long int	find_cd();
long int	find_cuts();
FILE	*open_out();
char	*clean_up();
char	*getcore();
char	*str_time();

struct key_match {		/* configuration of albums */
	int		km_file;		/* file number */
	long	km_rec;			/* record number */
	long	km_loc;		/* seek location for album */
	long	km_cut;		/* seek location for cut */
};

struct	keep {			/* albums being kept */
	char	*ka_name;	/* name of album record */
	char	*ka_art;	/* name of artist */
	short	ka_beg;		/* location of first cut */
	short	ka_end;		/* location of last cut */
};

struct	cuts {			/* collection of cuts */
	int     cut_nrec;	/* what album cut came from */
	int		cut_num;	/* what cut on album it is */
	char	*cut_name;	/* name of cut */
	short   cut_min;	/* minutes of cut time */
	short	cut_sec;	/* seconds of cut time */
	char	*cut_art;	/* cut artist */
};

struct	rec_cuts {		/* map cuts to albums */
	int	rc_beg;			/* first cut location */
	int	rc_end;			/* last cut location */
};

struct	tape {			/* one for each cut on tape */
	short	tape_rec;	/* position in album array */
	short	tape_cut;	/* cut on album */
	short	tape_sec;	/* time of cut in seconds */
	char	*t_cname;	/* name of cut */
	char	*t_cart;	/* name of artist */
};

struct	tapestat {		/* one for each tape side */
	char	*ts_name;	/* name of tape side */
	char	*ts_art;	/* artist of tape side */
	short	ts_num;		/* number of cuts on tape */
	short	ts_time;	/* total time on tape so far */
		/* now the tape cut structure */
	struct	tape	ts_tape[MAXTAPE];
};

struct	data_file	{		/* structure for data files */
	FILE	*df_fp;			/* file pointer */
	time_t	df_time;		/* last access time */
	char	*df_fname;		/* file name */
};

	/* real externals */

	/* sort of externals - EXTERN is commented out when called by define.c */
EXTERN	int		debugsw;				/* debugging switch (0=>off) */
EXTERN	char	*Prog;					/* program name (lower case) */

	/* structures for the actual database */
EXTERN	int		tot_dat;				/* number of index/data files */
EXTERN	long	tot_albums;				/* number of albums in database */
EXTERN	char	*mast_name;				/* master file name */
EXTERN	FILE	*mast_file;				/* master input file */
EXTERN	FILE	*art_file;				/* input artist key file */
EXTERN	FILE	*tit_file;				/* input title key file */
EXTERN	FILE	*cut_file;				/* input cut key file */
EXTERN	struct	data_file datafile[MAXNDX];	/* struct for data files */
EXTERN	int		max_open;				/* max. data files open */

	/* structures for the albums being kept */
	/* these are eligible for loading onto tape */
EXTERN int		tkeep;					/* total albums kept */
EXTERN struct	keep keep[MAXKEEP];		/* albums being kept */
EXTERN	int		tcuts;					/* total cuts in kept albums */
EXTERN	struct	cuts	cuts[MAXCUTS];	/* cuts of albums being kept */

	/* structures for the tape being made */
EXTERN	int		tapelen;				/* length of tape side */
EXTERN	int		tapeside;				/* current side of tape */
EXTERN	struct	tapestat tapes[MAXSIDES];	/* configuration of tapes */
