/*
 *				**** DISCOG ****
 *
 * (c) Copyright 1992 by William R. Pringle - all rights reserved
 *
 * This program produces a discography of an artist
 *	from the database used for the albums program.
 *
 */

#include <stdio.h>
#include <string.h>
#include <ctype.h>

#ifdef __MSDOS__
	/* MS-DOS */
#define	DIRSLASH	'\\'
#else
#define	DIRSLASH	'/'
#endif

main(argc,argv)
int		argc;
char	**argv;
{
	FILE	*fp = NULL;	/* input file */
	FILE	*ifp=NULL;	/* intro file pointer */
	FILE	*afp=NULL;	/* artist file pointer */
	char	*Prog;		/* program name */
	char	*p;			/* scanning pointer */
	char	mast[BUFSIZ];	/* database master file */
	char	intro[BUFSIZ];	/* intro file name */
	char	mastdir[BUFSIZ];	/* directory of master file */
	int		chkflg=1;		/* should we check w/ user on matches? */

	/* get program name (not full path) */
	if ((Prog = strrchr(argv[0],DIRSLASH)) == NULL) Prog = argv[0];
	else Prog++;
	argc--; argv++;		/* shift argument */

#ifdef __MSDOS__
	/* MS-DOS - convert program name to lower case */
	for (p = Prog; *p != '\0'; p++) if (isupper(*p)) *p = tolower(*p);
#endif

/* set up default database file */
		strcpy(mast,"albums.mst");		/* default name */
		strcpy(intro,"discog.int");		/* default name */

	while (argc > 0)
	{		/* process all arguments */
		if (argv[0][0] == '-') switch(argv[0][1])
		{		/* process option */
			case 'a':	/* file containing artist names */
				if (argv[0][2] != '\0') p = argv[0]+2;
				else
				{		/* file name is next argument */
					p = argv[1];		/* file  name */
					argc--; argv++;		/* skip -a flag */
				}

				/* p now points to file name */
				if ((afp = fopen(p,"r")) == NULL)
				{		/* bad file name */
					fprintf(stderr,"%s: Can't open artist file %s\n", Prog, p);
					exit (4);
				}
				break;

			case 'c':	/* don't check with user */
				chkflg = 0;
				break;

			case 'f':	/* master file name */
				if (argc < 2)
				{		/* missing argument */
					fprintf(stderr,"%s: No master file name given.\n", Prog);
					do_usage(Prog);
					exit(1);
				}
				if (argv[0][2] == '\0')
				{		/* file name is next argument */
					argc--; argv++;	/* get next argument */
					p = argv[0];	/* point to name */
				} else p = argv[0] + 2;	/* point to name */
				if (fp != NULL) fclose(fp);	/* close old file */
				strcpy(mast,p);		/* new master name */
				break;

			case 'i':	/* introductory text file */
				if (argc < 2)
				{		/* missing argument */
					fprintf(stderr,"%s: No intro file name given.\n", Prog);
					do_usage(Prog);
					exit(1);
				}
				if (argv[0][2] == '\0')
				{		/* file name is next argument */
					argc--; argv++;	/* get next argument */
					p = argv[0];	/* point to name */
				} else p = argv[0] + 2;	/* point to name */
				if (ifp != NULL) fclose(ifp);	/* close old file */
				strcpy(intro,p);		/* new master name */
				break;

			case 'h':	/* help */
				do_help(Prog);
				exit(0);

			default:	/* bad option */
				fprintf(stderr,"%s: Invalid option (%s)\n", Prog, argv[0]);
				do_usage(Prog);
				exit(1);
		} else
		{	/* no regular arguments supported */
			do_usage(Prog);
			exit(1);
		}
		argc--; argv++;		/* shift argument out */
	}

	/* arguments processed - set up */
	if ((fp = fopen(mast,"r")) == NULL)
	{		/* open failed */
		fprintf(stderr,"%s: Unable to open master file %s!\n", Prog, mast);
		exit(2);
	}

	if ((ifp = fopen(intro,"r")) == NULL)
	{		/* open failed */
		fprintf(stderr,"%s: Unable to open intro file %s!\n", Prog, intro);
		exit(2);
	}

	/* build directory name of master file */
	/* this is for building a file name if not in current directory */
	strcpy(mastdir,mast);				/* start with master file name */
	if ((p = strrchr(mastdir,DIRSLASH)) == NULL) p = strrchr(mastdir,'/');
	if (p == NULL) strcpy(mastdir,"./");
	else *(p+1) = '\0';

	discog(Prog,fp,ifp,afp,mastdir,chkflg);		/* produce discographies */

	fprintf(stderr,"%s: Normal termination.\n", Prog);
	return 0;
}

do_usage(Prog)
char	*Prog;	/* program name */
{		/* produce usage message */
	static	char	*msg[] = {
"Usage: %s [-a artist_file] [ -f master_file ] [-i intro_file] [-c]",
"	-OR- for help, type:",
"	%s -h",
NULL};

	prt_msg(msg,Prog);
}

do_help(Prog)
char	*Prog;
{		/* display help message */
	static	char	*msg[] = {
"This program produces discographies for artists",
"by reading database files used with the `albums' program.",
"The default albums master file is `albums.mst'",
"",
"If no artist file is provided, the user is prompted for artist names.",
"",
"Options:",
"\t-a afile\t`afile' is an artist file (see below)",
"\t-f mfile\t`mfile' is the master file (default albums.mst)",
"\t-i ifile\t`ifile' contains the intro text (default discog.int)",
"\t-c\t\tdon't check with user if an entry matches.",
"",
"The artist file has the format:",
"outfile  artist [ <tab> alternative_names]",
"Where `alternative_names' are other names involving the artist",
"\t(e.g., Lennon, John <tab> Beatles",
NULL};

	prt_msg(msg,Prog);
	do_usage(Prog);
}

prt_msg(msg,Prog)
char	*msg[];		/* list of messages */
char	*Prog;		/* program name */
{		/* display messages */
	while (*msg != NULL)
	{
		fprintf(stderr,*msg++,Prog);
		fputc('\n',stderr);
	}
}

discog(Prog,mfp,ifp,afp,mastdir,chkflg)
char	*Prog;		/* program name */
FILE	*mfp;		/* database master file */
FILE	*ifp;		/* intro master file */
FILE	*afp;		/* list of artists */
char	*mastdir;	/* directory of master file */
int		chkflg;		/* flag for checking with user on matches */
{		/* produce discographies */
	char	artname[BUFSIZ];	/* database file */
	char	*altart;			/* alternative artist name */
	char	fname[BUFSIZ];		/* output file name */
	FILE	*ofp;				/* output discog file */
	FILE	*xfp;				/* extra info file for artist */
	char	buf[BUFSIZ];		/* input buffer */
	char	*p;					/* handy dandy pointers */

	while (1)
	{		/* loop asking for artists */
		if (afp != NULL)
		{		/* we have list of artists, do them */
			if (fgets(fname,sizeof(buf),afp) == NULL) break;	/* EOF */

			/* got a real entry - file name followed by artist name */
			for (p=fname; !isspace(*p) && *p != '\0'; p++);	/* end of file name */
			if (*p == '\0')
			{		/* bad file format */
				fprintf(stderr,"%s: Invalid file name in list of artists (%s)\n",
						Prog, buf);
				continue;		/* ignore this line */
			}

			/* p points to end of file name */
			*p = '\0';		/* terminate file name */
			while (isspace(*(++p)));		/* find start of artist */
			if (*p == '\0')
			{		/* only file name */
				fprintf(stderr,"%s: Missing artist name in list of artists. (%s)\n", Prog, buf);
				continue;
			}

			/* p now points to name of artist - set things up */

			strcpy(artname,p);		/* copy artist name */
			for (p=artname; *p != '\0' && *p != '\n' && *p != '\r' && *p != '\t'; p++);
			*p = '\0';		/* terminate name */

			/* see if there are any alternative names (following a tab) */
			for (altart=artname; *altart != '\0' && *altart != '\t'; altart++);
			if (*altart != '\0') *altart++ = '\0';	/* split artist & alternative */

			printf("Building discography for: %s ", artname);
			if (*altart != '\0') printf("(%s)", altart);
			printf(" ...\n");
		} else
		{		/* interactive mode - ask user for name */
			printf("Enter artist name, '?' for help, or ENTER to quit: ");
			gets(artname);
			if (*artname == '\0') break;
			if (*artname == '?')
			{		/* asking for help */
				disc_help(Prog);
				continue;
			}
			for (altart =  artname; *altart != '\0' && *altart != '\t'; altart++);
			if (*altart == '\t') *altart++ = '\0';	/* terminate name & start alternative */

			printf("Enter name of output file (8 chars), or ENTER to cancel: ");
			gets(buf);
			strcpy(fname,buf);
			if (*fname == '\0') continue;	/* try again */
		}

		strcat(fname,".dsc");	/* build full file name */
		if (afp == NULL && access(fname,0) == 0)
		{		/* already there */
			printf("File %s already exists.  Overwrite? (y or n): ", fname);
			gets(buf);
			if (*buf != 'y') continue;	/* try again */
		}

		if ((ofp = fopen(fname,"w")) == NULL)
		{		/* can't open */
			fprintf(stderr,"%s: Can't open file %s!\n", Prog, fname);
			continue;		/* try again */
		}

		/* insert intro file at beginning of discog file */
		fprintf(ofp,"Discography for %s", artname);
		if (*altart != '\0') fprintf(ofp," (%s)", altart);
		fprintf(ofp,"\n\n");

		if (ifp != NULL)
		{
			rewind(ifp);
			while (fgets(buf,sizeof(buf),ifp) != NULL) fputs(buf,ofp);
			fputs("\n\n=============== Begin Discography ===============\n\n",
					ofp);
		}

		/* all ready to crank out entries */
		bld_disc(Prog,mfp,ofp,artname,mastdir,chkflg);	/* generate discography */

		/* add any extra information if it is given */
		if ((p = strrchr(fname,'.')) == NULL)
		{		/* problems */
			fprintf(stderr,"%s: Internal error!  Can find suffix for filename (%s)\n",
						Prog, fname);
			exit(1);
		}
		strcpy(p,".xtr");		/* name of extra info */
		if ((xfp = fopen(fname,"r")) != NULL)
		{		/* file opened */
			fputs("\n\n=============== Extra Information ===============\n\n",
				ofp);
			while (fgets(buf,sizeof(buf),xfp) != NULL) fputs(buf,ofp);
			fclose(xfp);
		}

		fclose(ofp);		/* clean up after ourselves */
	}
}

disc_help(Prog)
{		/* help with operation */
	static	char *msg[] = {
"",
"This program will create discographies for artists or groups.",
"You are being asked to enter the name of the artist or group.",
"The program will then search the database for entries containing",
"the name you enter.  The search will be case insensitive, and will",
"find the string anywhere in the artist field.",
"",
"You can specify alternative names after the search string by inserting",
"a tab character followed by the search string.  For example:",
"	Clegg, Johnny <tab> Jaluka or Savuka",
"The program will only search for `Clegg, Johnny'.  The alternative names",
"will be printed on the discography file.  If you use this string,",
"make sure that `Clegg, Johnny' in the artist field of all entries:",
"",
"	Clegg, Johnny~Juluka|Best of Juluka, The|1991|Rhythm Safari|CDL 57138",
"",
"Each time a match is found, you will be asked if it should",
"be included into the discography.  If you enter 'y', it will.",
"",
NULL};

	prt_msg(msg,Prog);
}
