/*
 *				**** DUMP RESTORE ****
 *
 * (c) Copyright 1992 by William R. Pringle - all rights reserved
 *
 * This file contains routines to save and restore the current configuration
 *
 */

#include <stdio.h>
#include "cd.h"

#define LOAD_FLDS	10

dump_config()
{
	FILE	*fp;					/* output file name */
	char	*p, buf[BUFSIZ];		/* user dialogue stuff */
	char	*flds[LOAD_FLDS];		/* for outputting strings */
	int		i, j;					/* misc stuff */

		/* first, get name of save file */
	printf("\nEnter base name (1-8 chars) of Save File: ");
	gets(buf);						/* get output file name */
	p = clean_up(buf);				/* get rid of garbage chars */
	if (p == EOS) return;			/* no name given */
	else if (strcmp(p,"*"))
	{		/* file name given, try to open it */
		fp = open_out(p,".dmp",1);	/* open output file */
		if (fp == NULL) return;		/* request cancelled */
	} else fp = stdout;				/* output to screen */

	/* output file opened, write stuff out */

		/* output identification info. compared when loading */
	fprintf(fp,"*\n* This file is a load file for the CD program.\n*\n");
	fprintf(fp,"* You should probably not play around with it.\n*\n");
	fprintf(fp,"*\n* tot_dat, tot_album,MAXSIDES,mast_name\n");
	fprintf(fp,"%d|%d|", tot_dat, tot_albums);
	fprintf(fp,"%d|%s\n", MAXSIDES, mast_name);

		/* output current limits */
	fprintf(fp,"* tkeep, tcuts, tapelen,tapeside\n");
	fprintf(fp,"%d|%d|%d|%d\n", tkeep, tcuts, tapelen,tapeside);

		/* now write out kept albums */
	for (i=0; i < tkeep; i++)
	{		/* loop thru kept albums */
		fprintf(fp,"* Kept Album %d:\n",i);
		fprintf(fp,"* ka_name, ka_art, ka_beg, ka_end\n");
		fprintf(fp,"%s|%s|%d|%d\n", keep[i].ka_name, keep[i].ka_art,
									keep[i].ka_beg,  keep[i].ka_end);
		fprintf(fp,"* Cuts: #, name, min, sec\n");
		for (j=keep[i].ka_beg; j < keep[i].ka_end; j++)
		{		/* loop thru cuts for this kept album */
			fprintf(fp,"\t%d|%s|%d:%2.2d|", j, cuts[j].cut_name,
										cuts[j].cut_min, cuts[j].cut_sec);
			if (cuts[j].cut_art == NULL) fprintf(fp,"\n");
			else fprintf(fp,"%s\n", cuts[j].cut_art);	/* cut artist */
		}		/* end loop thru cuts */
	}			/* end loop thru kept albums */

		/* all kept albums written out */
	fprintf(fp,"*\n* Tape Configuration\n*\n");		/* signal end of kept albums */
		/* now write out tape configuration */

	for (i=0; i < MAXSIDES; i++)
	{		/* loop thru tape sides */

		if (tapes[i].ts_name == NULL) flds[0] = "";
		else flds[0] = tapes[i].ts_name;

		if (tapes[i].ts_art == NULL) flds[1] = "";
		else flds[1] = tapes[i].ts_art;

		fprintf(fp,"* Tape Side %d:\n* name, artist, number cuts, tot time\n", i);
		fprintf(fp,"%s|%s|%d|%d\n", flds[0], flds[1],
									tapes[i].ts_num,  tapes[i].ts_time);

		for (j=0; j < tapes[i].ts_num; j++)
		{		/* loop thru cuts taped */
			fprintf(fp,"\t%d|%d|%d|%s|",
				tapes[i].ts_tape[j].tape_rec, tapes[i].ts_tape[j].tape_cut,
				tapes[i].ts_tape[j].tape_sec, tapes[i].ts_tape[j].t_cname);
			if (tapes[i].ts_tape[j].t_cart == NULL) fprintf(fp,"\n");
			else fprintf(fp,"%s\n", tapes[i].ts_tape[j].t_cart);
		}		/* end loop thru cuts for tape side */
		fprintf(fp,"*\n* End Tape Side %d\n*\n",i);	/* signal end of tape side */
	}			/* end loop thru tape sides */
}

load_config()
{
	char	*p, buf[BUFSIZ], *flds[LOAD_FLDS];		/* char bufs */
	int		nflds, err;						/* for input fields */
	FILE	*fp;							/* input file */
	int		i, j, k;						/* loop variables */

	printf("\nEnter base name (1-8 chars) of Load file: ");
	gets(buf);
	p = clean_up(buf);
	if (p > buf) strcpy(buf,p);
	strcat(buf,".dmp");
	if (access(buf,0) != 0)
	{
		printf("\nFile %s doesn't exist.  Request cancelled.\n", buf);
		return;
	} else if ((fp = fopen(buf,FMODE_READ)) == NULL)
	{
		printf("\nUnable to open file %s.  Command cancelled.\n", buf);
		return;
	}

	/* we now have the file open, make sure limits are the same */

	if ((nflds = next_line(buf,sizeof(buf),fp,flds,LOAD_FLDS)) != 4)
	{
		printf("\nWrong number of fields (%d) in first line of input file.\n", nflds);
		printf("(Probably not a Load file.) Request Cancelled.\n");
		return;
	}

			/* right number of fields, now check values */
	err = 0;	/* so far, so good */
	if ((i = atoi(flds[0])) != tot_dat)
	{
		printf("Wrong number of data files (%d instead of %d)\n", i, tot_dat);
		err++;
	}
	if ((i = atoi(flds[1])) != tot_albums)
	{
		printf("Wrong number of albums (%d instead of %d)\n", i, tot_albums);
		err++;
	}
	if ((i = atoi(flds[2])) != MAXSIDES)
	{
		printf("Wrong number of Max. Tape Sides (%d instead of %d)\n", i, MAXSIDES);
		err++;
	}
	if (strcmp(flds[3],mast_name))
	{
		printf("Wrong master file name (%s instead of %s)\n", flds[3], mast_name);
		err++;
	}
	if (err > 0)
	{
		printf("\nCancel this request? (y or n): ");
		gets(buf);
		p = clean_up(buf);
		if (*p != 'n' && *p != 'n') return;
	}

	/* Same configuration, now get configuration limits */
	/* Any I/O errors after this point, we will have to die */
	/*	since we will have corrupted the program limits */

	printf("\nMaster file is %s\n", mast_name);
	printf("%6d data files in database.\n", tot_dat);
	printf("%6d albums in database.\n", tot_albums);
	printf("%6d possible sides to tape.\n", MAXSIDES);

		/* get & parse next line */
	if ((nflds = next_line(buf,sizeof(buf),fp,flds,LOAD_FLDS)) != 4)
	{
		printf("\nWrong number of fields (%d) in second line.\n", nflds);
		printf("\nRequest cancelled.\n");
		return;
	}
	tkeep = atoi(flds[0]);
	tcuts = atoi(flds[1]);
	tapelen = atoi(flds[2]);
	tapeside = atoi(flds[3]);

	printf("\n");
	printf("%6d albums kept.\n", tkeep);
	printf("%6d cuts in the kept albums.\n", tcuts);
	printf("Each tape side holds %s\n", str_time(tapelen));
	printf("Current tape side is %d\n", tapeside+1);

	/* configuration limits gotten, now read kept albums */

	for (i=0; i < tkeep; i++)
	{		/* loop thru kept albums */
		if ((nflds = next_line(buf,sizeof(buf),fp,flds,LOAD_FLDS)) != 4)
		{
			printf("\nWrong number of fields (%d) for Kept Album line.\n", nflds);
			zap_all();
			return;
		} else printf("Album %d:\n%s|%s\n", i+1, flds[0], flds[1]);

		keep[i].ka_name = getcore(flds[0]);
		keep[i].ka_art  = getcore(flds[1]);
		keep[i].ka_beg  = atoi(flds[2]);
		keep[i].ka_end  = atoi(flds[3]);

		for (j = keep[i].ka_beg; j < keep[i].ka_end; j++)
		{		/* loop thru cuts for this kept album */
			if ((nflds = next_line(buf,sizeof(buf),fp,flds,LOAD_FLDS)) != 4)
			{
				printf("\nWrong number of fields (%d) for Album Cut line.\n", nflds);
				zap_all();
				return;
			}
			if ((k=atoi(flds[0])) != j)
			{
				printf("\nWrong Cut Number (%d) found instead of %d.\n", k, j);
				printf("\nBad load file.  Clearing configuration.\n");
				zap_all();
				return;
			}
				/* display this cut */
			printf("\t%2d. %s|%s|%s\n", (j-keep[i].ka_beg)+1, flds[1], flds[2], flds[3]);

			cuts[j].cut_name = getcore(flds[1]);
			cuts[j].cut_min = parse_time(flds[2]);	/* total seconds */
			cuts[j].cut_sec = cuts[j].cut_min % 60;	/* get seconds */
			cuts[j].cut_min /= 60;					/* convert to minutes */
			if (*(flds[2]) != EOS) cuts[j].cut_art = getcore(flds[3]);
		}		/* end loop thru cuts of this kept album */
	}			/* end loop thru kept albums */

	/* kept albums now read in, get tape configuration */

	for (i=0; i < MAXSIDES; i++)
	{		/* loop thru all tape sides */
		if ((nflds = next_line(buf,sizeof(buf),fp,flds,LOAD_FLDS)) != 4)
		{		/* get tape side */
			printf("\nWrong number of fields (%d) for Tape Side Line.\n", nflds);
			printf("\nCorrupted Load File.\n");
			zap_all();
			return;
		}
			/* have tape side line */
		if (*(flds[0]) == EOS) tapes[i].ts_name = NULL;
		else tapes[i].ts_name = getcore(flds[0]);
		if (*(flds[1]) == EOS) tapes[i].ts_art = NULL;
		else tapes[i].ts_art = getcore(flds[1]);
		tapes[i].ts_num = atoi(flds[2]);
		tapes[i].ts_time = atoi(flds[3]);

		for (j=0; j < tapes[i].ts_num; j++)
		{		/* get all cuts on tape */
			if ((nflds = next_line(buf,sizeof(buf),fp,flds,LOAD_FLDS)) != 5)
			{
				printf("\nWrong number of fields (%d) in Tape Cut Line.\n", nflds);
				printf("Corrupted Load File.\n");
				zap_all();
				return;
			}

			tapes[i].ts_tape[j].tape_rec = atoi(flds[0]);
			tapes[i].ts_tape[j].tape_cut = atoi(flds[1]);
			tapes[i].ts_tape[j].tape_sec = atoi(flds[2]);
			tapes[i].ts_tape[j].t_cname  = getcore(flds[3]);
			if (*(flds[4]) == EOS) tapes[i].ts_tape[j].t_cart = NULL;
			else tapes[i].ts_tape[j].t_cart = getcore(flds[4]);
		}		/* end loop thru cuts on this tape side */
	}			/* end loop thru tape sides */
}

next_line(buf,len,fp,flds,maxflds)
char	*buf;					/* input buffer */
unsigned len;					/* size of buf */
FILE	*fp;					/* input file to read */
char	*flds[];				/* input fields for parsing */
int		maxflds;				/* max. number of fields to parse */
{
	int	nflds;					/* number of fields that are parsed */
	char	*p;					/* for parsing */

	while (1)
	{		/* loop until we find a non-comment line */
		if (fgets(buf,len,fp) == NULL)
		{
			nflds = -1;		/* got EOF or Error */
			break;			/* exit */
		} else
		{
			p = clean_up(buf);
			if (*p == '*') continue;	/* comment line, ignore */
			if (p > buf) strcpy(buf,p);
			nflds = split_flds(buf,flds,'|',0,maxflds);
			break;	/* we have a non-comment line, go with it */
		}
	}
	return nflds;
}
