/*
 *
 * (c) Copyright 1992 by William R. Pringle - all rights reserved
 *
 *				**** FIND CDS ****
 *
 * EMACS_MODES: c, tabstop=4, !lnumb, !fill
 *
 * This routine will read through the specified input file and find
 *	all album description lines (not starting w/ white space)
 *
 * For each album line:
 *	key records are written for the title, artist, and cut fields.
 *	Multiply key records are formed by using a tilde (~)
 *		(e.g., Ten Thousand Maniacs~10,000 Maniacs)
 *
 * A key record is of the format:
 *		key_str | file # | rec_no | loc
 *	where
 *		key_str		is the key string built from the field
 *		file #		is the number of the file
 *		rec_no		is the album number (starting w/ 1 for each file)
 *		loc			is the seek location (in bytes from start of file)
 *
 *
 * There is a key record file for each field (artist, title, & cut)
 *
 */

#include <stdio.h>
#include <string.h>
#include "cd.h"

find_cds(fname,fnumb,keyart,keytit,keycut)
char *fname;		/* input file name */
int	fnumb;			/* file number */
FILE *keyart;		/* artist key file */
FILE *keytit;		/* title key file */
FILE *keycut;		/* cut key file */
{

	char	buf[BUFSIZ];		/* input buffer */
	char	*p;					/* pointer variable */

	char	*flds[DATFLDS];		/* array for record fields */
	int		nflds;				/* how many fields were found */
	char	*art_flds[SPLITFLDS];	/* fields for multiple artists */
	int		nart_flds;			/* number of artists */
	char	*key;				/* ptr for key field */
	int		cnt;				/* album counter for file */
	long	loc;				/* location of record in file */
	int		i;					/* loop variable */

		/* file pointers */
	FILE	*infile;			/* input file pointer */

	strcpy(buf,fname);			/* copy name of file */
	p = strrchr(buf,'.');		/* find suffix */
	if (p == NULL) p = buf + strlen(buf);

	strcpy(p,".alb");			/* build data file name */
	if ((infile=fopen(buf,FMODE_READ)) == NULL) return 0L;

	*p = EOS;					/* truncate to base name */

	cnt = 0;
	loc = 0L;

	while ((loc = find_cd(infile,buf)) >= 0L)
	{		/* album line found, parse it */
		if (debugsw) fprintf(stderr,"%s: file %d, loc %ld, album %d \n>>%s<<\n",
						Prog, fnumb, loc, cnt, buf);

		if (*buf == EOS) continue;
		cnt++;

		if ((nflds = split_flds(buf,flds,'|',0,DATFLDS)) != DATFLDS)
		{		/* hmmm ... wrong number of fields */
			fprintf(stderr,"%s: Album %d for file %d (%s) has %d fields instead of %d\n",
								Prog, cnt, fnumb, fname, nflds, DATFLDS);
			for (i=0; i < nflds; i++) printf("flds[%d] >>%s<<\n", i, flds[i]);
			continue;
		}

		if (nflds > DAT_ART)
		{			/* we have at least one artist - output key record(s) */
			nart_flds = split_flds(flds[DAT_ART],art_flds,'~',0,SPLITFLDS);
			for (i=0; i < nart_flds; i++)
			{
				if (stricmp(flds[i],"various") == 0) continue;	/* ignore various */
				key = get_key(art_flds[i]);		/* key for artist */
				fprintf(keyart,KEYFMT, key, fnumb, cnt, loc,loc);	/* key, file, & record */
			}
		}

		if (nflds > DAT_TIT)
		{			/* we do have a title, output key record */
			nart_flds = split_flds(flds[DAT_TIT],art_flds,'~',0,SPLITFLDS);
			for (i=0; i < nart_flds; i++)
			{
				key = get_key(art_flds[i]);		/* key for title */
				fprintf(keytit,KEYFMT, key, fnumb, cnt, loc,loc);	/* key, file, & record */
			}
		}

		if ((loc = find_cuts(infile,keyart,keytit,keycut,fnumb,cnt,loc)) > 0)
		{		/* found an album record, back up to just before it */
			fseek(infile,loc,SEEK_SET);
		}
	}

	fclose(infile);
	return cnt;
}
