/*
 *
 * (c) Copyright 1992 by William R. Pringle - all rights reserved
 *
 *				**** FIND CUTS ****
 *
 * This routine finds all cuts in database and writes a record for them
 *
 * Both a cut key record and an artist key record (if needed) are written
 *
 */

#include <stdio.h>
#include "cd.h"

static	long total_cuts=0;					/* total number of cuts found */

long int
find_cuts(infile,keyart,keytit,keycut,fnumb,cnt,aloc)
FILE	*infile;						/* input file */
FILE	*keyart;						/* artist key file */
FILE	*keytit;						/* title key file */
FILE	*keycut;						/* cut key file */
int		fnumb;							/* file number */
int		cnt;							/* album number */
long	aloc;							/* album rec location */
{		/* write out cuts, return location of next album record */
	long	loc;						/* location in file */
	char	buf[BUFSIZ];				/* input buffer */
	char	*cut_flds[CUTFLDS];			/* fields for cut record */
	int		nflds;						/* number of fields found */
	char	*sub_flds[SPLITFLDS];		/* fields for subfields */
	int		nsubflds;					/* number of subfields found */
	int		i;							/* loop variable */
	char	*key;						/* hash key */
	char	*p;							/* parsing ptr */

	if (infile == NULL || keyart == NULL || keytit == NULL || keycut == NULL)
		fprintf(stderr,"\nNull File pointer passed to find_cuts()\n");
	if (fnumb < 0 || cnt < 0)
		fprintf(stderr,"\nNegative count passed to find_cuts()\n");

	loc = ftell(infile);				/* save location before read */

	while (fgets(buf,BUFSIZ,infile) != NULL)
	{		/* keep reading input file until an album or EOF */
			/* clean up input record before we look at it */
		for (p = buf; *p != '\0' && *p != '\n' && *p != '\r'; p++);
		*p = '\0';

		if (*buf == '\0')
		{			/* ignore empty lines */
			loc = ftell(infile);
			continue;
		} else if (*buf != '\t') return loc;		/* album - return loc before */

			/* we have a cut record - process it */
			/* actual data starts after leading tab */
		nflds = split_flds(buf+1,cut_flds,'|',0,CUTFLDS);
		if (nflds > CUT_NAME)
		{		/* name there - output it */
			nsubflds = split_flds(cut_flds[CUT_NAME],sub_flds,'~',0,SPLITFLDS);
			for (i=0; i < nsubflds; i++)
			{		/* loop thru subfields of cut name */
				key = get_key(sub_flds[i]);		/* key for cut name */
				if (*key == '\0') continue;		/* skip null keys */
				fprintf(keycut,KEYFMT,key,fnumb,cnt,aloc,loc);	/* output key */
				total_cuts++;					/* increment count of cuts */
			}
		}

		if (nflds > CUT_ART)
		{		/* guest artist - output it, too */
			nsubflds = split_flds(cut_flds[CUT_ART],sub_flds,'~',0,SPLITFLDS);
			for (i=0; i < nsubflds; i++)
			{		/* loop thru subfields of guest artist name */
				key = get_key(sub_flds[i]);		/* key for cut name */
				fprintf(keyart,KEYFMT,key,fnumb,cnt,aloc,loc);	/* output key */
			}
		}
		loc = ftell(infile);
	}
	return loc;
}

void
tot_cuts(keycut)
FILE	*keycut;		/* key file for cut */
{		/* output total cuts */
	fprintf(keycut,KEYFMT,"",0,0,total_cuts,total_cuts);
	printf("%ld total cuts found.\n", total_cuts);
}
