/*
 *
 * (c) Copyright 1992 by William R. Pringle - all rights reserved
 *
 *				**** FIND KEY ****
 *
 * EMACS_MODES: c, !fill, tabstop=4, !lnumb
 *
 * This routine matches a key string to the key file
 *
 * Input parameters:
 *	infile		file pointer
 *	str			key string to be found
 *	matches[]	output key record locations
 *
 * Return values:
 *	number of matches found (function value)
 *	key record locations (in array)
 *
 */

#include <stdio.h>
#include "cd.h"

find_key(infile,key,matches,maxmatch)
FILE	*infile;	/* input file pointer */
char	*key;		/* input key string */
struct	key_match	matches[];	/* output key records */
int		maxmatch;	/* maximum matches to return */
{
	int		nmatch;			/* number of matches found */
	int		rec_count;		/* record counter */
	long	max_rec;		/* maximum records */
	char	buf[BUFSIZ];	/* input file */
	int		nflds;			/* number of fields found */

	char	*flds[KEYFLDS];	/* fields to be found */
	int		lkey;			/* length of key to compare */

	long atol();			/* convert string to long integer */

	rewind(infile);		/* start at beginning */
	max_rec = 0L;
	rec_count = -1;		/* no records yet */
	nmatch = 0;			/* nothing found yet */

	lkey = strlen(key);	/* length to compare */
	if (lkey > KEYLEN) lkey = KEYLEN;	/* max. length in  file */

	while ((fgets(buf,sizeof(buf),infile)) != NULL)
	{
		rec_count++;
			/* check for match */
		if ((nflds = split_flds(buf,flds,'|',0,KEYFLDS)) != KEYFLDS)
		{
			fprintf(stderr,"find_key: key record %d contains %d fields\n",
					rec_count, nflds);
			fprintf(stderr,"\t(should be %d) >>%s<<\n", nflds, buf);
			continue;
		}

		if (rec_count == 0)
		{
			max_rec = atol(flds[KEY_LOC]);
			continue;
		} else if (strncmp(key,flds[KEY_KEY],lkey) < 0) return nmatch;
		else if (strncmp(key,flds[KEY_KEY],lkey) == 0)
		{		/* we have a match, set up things */
			matches[nmatch].km_file = atol(flds[KEY_FILE]);	/* file number */
			matches[nmatch].km_rec = atol(flds[KEY_REC]);		/* record number */
			if (matches[nmatch].km_rec > max_rec)
			{
				fprintf(stderr,"Record number too large (%ld)",
					matches[nmatch].km_rec);
				fprintf(stderr," in line %d of sorted key file.\n", rec_count);
			}
			matches[nmatch].km_loc = atol(flds[KEY_LOC]);		/* record location */
			matches[nmatch].km_cut = atol(flds[KEY_CUT]);		/* location of cut */
			if (++nmatch >= maxmatch) return nmatch;
		}
	}
}
