/*
 *
 * (c) Copyright 1992 by William R. Pringle - all rights reserved
 *
 *				**** MAKE TAPE ****
 *
 * This file contains routines to build a tape from CDs
 *
 */

#include <stdio.h>
#include "cd.h"

make_tape()
{
	int		option;			/* command option */
	char	buf[BUFSIZ];	/* input buffer */
	char	*p;				/* handy dandy pointer variable */
	int		tmp;			/* temporary tape length */

	while (1)
	{		/* loop until an exit option */
		printf("\n");
		tape_used(stdout,tapeside);			/* our current status */
		printf("Options:\n");
		printf("L\tList Cuts on Tape\n");
		printf("A\tAdd Cuts to Tape\n");
		printf("D\tDelete Cuts from Tape\n");
		printf("M\tMove Cuts on Tape\n");
		printf("E\tEdit Cuts on Tape\n");
		printf("F\tFind Cuts that Fit\n");
		printf("N\tChange to New Side of Tape\n");
		printf("T\tChange Tape length\n");
		printf("\n");
		printf("K\tKeep more Albums\n");
		printf("Z\tZap all tape sides\n");
		printf("W\tWrite Tape Configuration\n");
		printf("\n");
		printf("?\tHelp Message\n");
		printf("Q\tReturn to Main Program\n");
		printf("X\tExit Program\n");
		printf("\nEnter Option: ");
		gets(buf);
		p = clean_up(buf);

		if (*p == EOS) continue;
		option = *p;

		switch(option) {

				/* add cuts to tape */
		case 'a':
		case 'A':
			add_tape();
			break;

				/* delete cuts from tape */
		case 'd':
		case 'D':
			del_tape();
			break;

		case 'e':	/* edit cuts on tape */
		case 'E':
			edit_tape();
			break;

		case 'f':	/* find cuts that fit */
		case 'F':
			fit_tape();
			break;

		case 'k':	/* keep albums */
		case 'K':
			option = 'K';		/* album menu */
			return option;
		/***** No break needed since this case returns ****/

				/* list option */
		case 'l':
		case 'L':
			list_tape();
			break;

				/* move cuts on tape */
		case 'm':
		case 'M':
			move_tape();
			break;

				/* change tape sides */
		case 'n':
		case 'N':
			print_side();
			printf("Enter new side (1 to %d): ", MAXSIDES);
			gets(buf);
			p = clean_up(buf);
			if (*p == EOS)
			{
				printf("Tape side unchanged\n");
				continue;
			}
			tmp = atoi(p);		/* convert to side number */
			if (tmp < 1 || tmp > MAXSIDES)
			{
				printf("Invalid side (%s).\n", buf);
				continue;
			}
			tapeside = tmp -1;			/* change the tape side */
			print_side();
			break;

				/* change tape length */
		case 't':
		case 'T':
			tape_len();
			printf("Enter new length in minutes, or ENTER to cancel: ");
			gets(buf);
			p = clean_up(buf);
			if (*p != EOS)
			{
				if ((tmp = atoi(p)) < 1)
				{
					printf("Invalid time (%s) ignored\n", buf);
				} else {
					tapelen = tmp*60/2;
					tape_len();
				}
			} else printf("Command cancelled\n");
			break;

				/* save tape configuration */
		case 'w':
		case 'W':
			write_tape();
			break;

		case 'q':	/* return to main routine */
		case 'Q':
		case 'x':	/* exit program */
		case 'X':
			return option;

		case 'z':	/* zap tape sides */
		case 'Z':
			zap_tape(1);
			break;

		case '?':	/* help */
			help("TAPE");
			break;

		default:	/* bad command */
			fprintf(stderr,"Invalid command option (%s)\n", buf);
			break;
		}		/* end switch on command option */
	}			/* end option loop */
}

print_side()
{
	printf("\nCurrent tape side is %d\n", tapeside+1);
}

tape_len()
{
	printf("Current tape is %d minutes (%d per side)\n", tapelen/30, tapelen/60);
}

list_tape()
{
	char	buf[BUFSIZ];

	list_albums(stdout,0);
	ontape(stdout,tapeside);
	printf("Press ENTER to continue: ");
	gets(buf);
}

ontape(fp,side)
FILE	*fp;	/* output file */
int		side;	/* side of tape to print */
{
	int		i;		/* loop variable */

	tape_used(fp,side);
	for (i=0; i < tapes[side].ts_num; i++)
		list_tcut(i,fp,&tapes[side].ts_tape[i]);
}


list_tcut(cut,fp,t)
int		cut;			/* cut (zero base) */
FILE	*fp;			/* output file */
struct	tape	*t;		/* entry to be printed */
{
	fprintf(fp,"%3d. %2d %2d %s %s\n",
		cut+1,						/* cut number */
		t->tape_rec+1,				/* album number */
		t->tape_cut+1,				/* cut on album */
		str_time(t->tape_sec),		/* time of cut */
		t->t_cname);				/* name of cut */
}

cut_trail(fp)
FILE	*fp;			/* output file */
{		/* print trailer for cut list */
/***************123. 12 12 HH:MM ssssssss ***************/
	fprintf(fp," |   |  |  |     |\n");
	fprintf(fp," |   |  |  |     -- Name of cut\n");
	fprintf(fp," |   |  |  --- Time of cut\n");
	fprintf(fp," |   |  --- Cut number on album\n");
	fprintf(fp," |   --- Album number (see top of page)\n");
	fprintf(fp," --- Sequence number on tape\n");
}


add_tape()
{		/* add cuts to the current tape side */
	char	buf[BUFSIZ], *p;
	int		nrec;			/* album number */
	int		fcut;			/* location first cut in cuts */
	int		mcut;			/* how many cuts on current album */
	int		ncuts, cut_beg[MAXTAPE], cut_end[MAXTAPE];	/* cut ranges */
	int		ncut, cutbeg, cutend;		/* cut number values */
	int		i, j;		/* good old i and friends */
	int		looping;

	ncut = tapes[tapeside].ts_num;

	for (looping = 1; looping > 0;)
	{		/* loop until done entering cuts */
		tape_used(stdout,tapeside);
		if (tkeep > 1)
		{
			printf("\nEnter position number of album or ENTER to end: ");
			gets(buf);
			p = clean_up(buf);
			if (*p == EOS) break;
			nrec = atoi(p);			/* convert album number */
			if (nrec < 1 || nrec > tkeep)
			{
				printf("Invalid album position (%s).  Try again.\n", buf);
				continue;
			} else nrec--;			/* convert to displacement */
		} else nrec = 0;			/* only album to add */

		list_album(stdout,nrec,1);		/* list title & cuts */
		fcut = keep[nrec].ka_beg;		/* loc of first cut */
		mcut = keep[nrec].ka_end - fcut;	/* how many cuts */

		if (mcut < 1)
		{
			printf("Album has no cuts defined.\n");
			printf("You will have to enter the information manually.\n");
			printf("I will ask you for the information as I need it.\n");
			printf("\nEnter cut number(s) to tape: ");
		} else printf("Enter cut number(s) (1 to %d): ", mcut);

		gets(buf);
		p = clean_up(buf);
		if (*p == EOS)
		{
			looping = 0;		/* terminate loop */
			continue;
		}
		ncuts = parse_nums(p,cut_beg,cut_end,MAXTAPE);

			/* loop thru cut ranges: validate & store */
		for (i=0; i < ncuts; i++)
		{		/* for each range */
			cutbeg = cut_beg[i];
			cutend = cut_end[i];
			if (mcut > 0)
			{
				if (cutbeg < 1 || cutbeg > mcut || cutend < cutbeg || cutend > mcut)
				{
					printf("Invalid range #%d (%d to %d)\n", i, cutbeg, cutend);
					continue;
				}
			}
				/* convert to displacement */
			cutbeg--;
			cutend--;

			for (j=cutbeg; j <= cutend; j++)
			{		/* loop thru all cuts */
				if (ncut >= MAXTAPE)
				{
					fprintf(stderr,"You can only store %d cuts per tape\n", MAXTAPE);
					break;
				}

				if (mcut > 0) add_cut(tapeside,nrec,ncut,fcut,j);
				else if (!new_cut(tapeside,nrec,ncut,j)) continue;

				/* entry added - see if first cut on tape */

				if (ncut == 0) first_cut(tapeside,nrec,ncut);

				tapes[tapeside].ts_time += tapes[tapeside].ts_tape[ncut].tape_sec;
				tapes[tapeside].ts_num = ++ncut;	/* on to next cut */
			}		/* end loop thru cuts */
		}			/* end loop thru cut ranges */
		if (tkeep == 1) looping = 0;		/* only one album to tape */
	}				/* end loop for cuts */
}

add_cut(tapeside,nrec,ncut,fcut,acut)
int		tapeside;		/* what side of the tape */
int		nrec;			/* what album is being selected */
int		ncut;			/* what cut on the tape */
int		fcut;			/* where first cut of album is located */
int		acut;			/* what cut to add */
{		/* we have a live one, save it */
	char	buf[BUFSIZ], *p;
	int		k;			/* time in seconds */
	tapes[tapeside].ts_tape[ncut].tape_rec = nrec;
	tapes[tapeside].ts_tape[ncut].tape_cut = acut;
	k = cuts[fcut+acut].cut_min * 60 + cuts[fcut+acut].cut_sec;
	if (k < 1)
	{		/* no time for cut */
		printf("No time given for cut %d on album %d.  Enter in MM:SS format: ", ncut+1, nrec+1);
		gets(buf);
		p = clean_up(buf);
		k = parse_time(p);
	}
	tapes[tapeside].ts_tape[ncut].tape_sec = k;
	tapes[tapeside].ts_tape[ncut].t_cname = getcore(cuts[fcut+acut].cut_name);
	if (cuts[fcut+acut].cut_art == NULL)
		tapes[tapeside].ts_tape[ncut].t_cart = getcore(keep[nrec].ka_art);
	else
		tapes[tapeside].ts_tape[ncut].t_cart = getcore(cuts[fcut+acut].cut_art);
}


new_cut(tapeside,nrec,ncut,acut)
int		tapeside;		/* what side of tape we are on */
int		nrec;			/* what album is being added */
int		ncut;			/* the number of this cut on tape */
int		acut;				/* what cut on this album to be added */
{		/* album doesn't have cuts */
	char	buf[BUFSIZ], *p;
	tapes[tapeside].ts_tape[ncut].tape_rec = nrec;
	tapes[tapeside].ts_tape[ncut].tape_cut = acut;
	printf("Enter name of cut %s: ");
	gets(buf);
	p = clean_up(buf);
	if (*p == EOS) return 0;
	tapes[tapeside].ts_tape[ncut].t_cname = getcore(p);
	printf("Enter Artist for cut %s: ");
	gets(buf);
	p = clean_up(buf);
	if (*p == EOS) return 0;
	tapes[tapeside].ts_tape[ncut].t_cart = getcore(p);
	printf("Enter time for cut %s: ");
	gets(buf);
	if ((p = clean_up(buf)) == NULL) return 0;
	tapes[tapeside].ts_tape[ncut].tape_sec = parse_time(p);
	return 1;		/* signal new cut defined OK */
}   	/* cut stored  - add to tape totals */

first_cut(tapeside,nrec,ncut)
int		tapeside;		/* what side of tape is being created */
int		nrec;			/* what album is the first cut from */
int		ncut;			/* what cut on tape is being added */
{		/* first time for tape side - get artist & title */
	char	buf[BUFSIZ], *p, *q;

	q = tapes[tapeside].ts_tape[ncut].t_cart;
	if (q == NULL) q = keep[nrec].ka_art;
	printf("Enter ARTIST for tape (default %s, ? for `Various')\n", q);
	gets(buf);
	p = clean_up(buf);
	if (*p == EOS) p = q;
	else if (*p == '?') p = "Various";
	tapes[tapeside].ts_art = getcore(p);

	q = keep[nrec].ka_name;		/* default tape name */
	printf("Enter TITLE for tape side (default %s, ? for `Various')\n", q);
	gets(buf);
	p = clean_up(buf);
	if (*p == EOS) p = q;
	else if (*p == '?') p = "Various";
	tapes[tapeside].ts_name = getcore(p);
}


tape_used(fp,side)
FILE	*fp;
int	side;
{
	int left;

	fprintf(fp,"\nSide %d of tape has %d cuts for %s so far.",
				side+1, tapes[side].ts_num, str_time(tapes[side].ts_time));

	left = tapelen - tapes[side].ts_time;
	if (left > 0) fprintf(fp," (%s remaining)\n", str_time(left));
	else if (left == 0) fprintf(fp," - Perfect fit!\n");
	else fprintf(fp," *** over by %s\n", str_time(-left));

	if (fp != stdout)
		fprintf(fp,"\t(See bottom of page for column descriptions)\n");
}

del_tape()
{		/* delete a cut from the tape */
	int		i, j;			/* loop variable */
	int		delpos;			/* position to delete */
	int		delcnt, delbeg[MAXCUTS], delend[MAXCUTS];
	int		deltime;		/* time of cut to be deleted */
	int		maxcut;			/* maximum cut found after delete */
	char	buf[BUFSIZ];	/* input buffer */
	char	*p;				/* temp ptr variable */

	tape_used(stdout,tapeside);
	if (tapes[tapeside].ts_num <= 1)
	{
		printf("Nothing to delete.  Command cancelled.\n");
		return;
	} else ontape(stdout,tapeside);

	printf("Enter cut to delete (1 to %d) or ENTER to cancel: ",
			tapes[tapeside].ts_num);
	gets(buf);
	p = clean_up(buf);
	if (*p == EOS) return;
	if ((delcnt = parse_nums(p,delbeg,delend,MAXCUTS)) < 1) return;
	for (i=0; i < delcnt; i++)
	{		/* loop thru ranges given */
		for (j=delbeg[i]; j <= delend[i]; j++)
		{
			if (j < 1 || j > tapes[tapeside].ts_num)
			{
				printf("Invalid range (%d to %d)\n", delbeg[i], delend[i]);
				return;
			}
			list_tcut(j-1,stdout,&tapes[tapeside].ts_tape[j-1]);
		}
	}

	printf("Delete? (y or n): ");
	gets(buf);
	p = clean_up(buf);
	if (*p != 'y' && *p != 'Y')
	{
		printf("Delete cancelled.\n");
		return;
	}

		/* loop thru cuts and delete */
	deltime = 0;			/* no time deleted yet */
	for (i=0; i < delcnt; i++)
	{
		for (j = delbeg[i]; j <= delend[i]; j++)
		{		/* clear deleted entry */
			delpos = j-1;
			if (tapes[tapeside].ts_tape[delpos].t_cname != NULL)
			{
				free(tapes[tapeside].ts_tape[delpos].t_cname);
				tapes[tapeside].ts_tape[delpos].t_cname = NULL;
			}
			if (tapes[tapeside].ts_tape[delpos].t_cart != NULL)
			{
				free(tapes[tapeside].ts_tape[delpos].t_cart);
				tapes[tapeside].ts_tape[delpos].t_cart = NULL;
			}
			deltime += tapes[tapeside].ts_tape[delpos].tape_sec;	/* remember time */
			tapes[tapeside].ts_tape[delpos].tape_sec = -1;
		}
	}

		/* loop thru tape side and see what's left */
	maxcut = 0;			/* no good cuts found yet */
	for (i=0; i < tapes[tapeside].ts_num; i++)
	{
		if (tapes[tapeside].ts_tape[i].tape_sec < 0) continue;	/* deleted */
			/* this cut is good.  If any have been deleted, move it up */
		if (maxcut == i)
		{		/* no deletes yet, just increment counter */
			maxcut++;
			continue;
		}
			/* delete has occurred, move this guy up */
		tapes[tapeside].ts_tape[maxcut].tape_rec = tapes[tapeside].ts_tape[i].tape_rec;
		tapes[tapeside].ts_tape[maxcut].tape_cut = tapes[tapeside].ts_tape[i].tape_cut;
		tapes[tapeside].ts_tape[maxcut].tape_sec = tapes[tapeside].ts_tape[i].tape_sec;
		tapes[tapeside].ts_tape[maxcut].t_cname  = tapes[tapeside].ts_tape[i].t_cname;
		tapes[tapeside].ts_tape[maxcut].t_cart   = tapes[tapeside].ts_tape[i].t_cart;
		maxcut++;			/* one more cut */

			/* now clean out old position */
		tapes[tapeside].ts_tape[i].tape_rec = -1;
		tapes[tapeside].ts_tape[i].tape_cut = -1;
		tapes[tapeside].ts_tape[i].tape_sec = 0;
		tapes[tapeside].ts_tape[i].t_cname  = NULL;
		tapes[tapeside].ts_tape[i].t_cart   = NULL;
	}
			/* finally, remove cuts from tape totals */
	tapes[tapeside].ts_num = maxcut;		/* one less cut */
	tapes[tapeside].ts_time -= deltime;	/* remove time of cut */
}

move_tape()
{
	int		maxcut;				/* how many cuts on tape */
	int		oldloc, newloc;		/* from and to */
	int		i;
	char	buf[BUFSIZ];		/* input buffer */
	char	*p;					/* temp ptr var */
	struct	tape	tapesave;	/* temporary area for swapping */

	if ((maxcut = tapes[tapeside].ts_num) < 2)
	{
		printf("Nothing to move.\n");
		return;
	} else ontape(stdout,tapeside);

	printf("Enter position (1 to %d) of cut to move or ENTER to cancel: ", maxcut);
	gets(buf);
	p = clean_up(buf);
	if (*p == EOS) return;
	oldloc = atoi(p);
	if (oldloc < 1 || oldloc > maxcut)
	{
		printf("Invalid position.  Command cancelled.\n");
		return;
	} else oldloc--;				/* convert to displacement */
	list_tcut(oldloc,stdout,&tapes[tapeside].ts_tape[oldloc]);

	printf("Enter position (1 to %d) of where to move cut %d or ENTER to cancel: ", maxcut, oldloc+1);
	gets(buf);
	p = clean_up(buf);
	newloc = atoi(p);
	if (newloc < 1 || newloc > maxcut)
	{
		printf("Invalid position.  Command cancelled.\n");
		return;
	} else newloc--;

	if (oldloc == newloc)
	{
		printf("It's already there.  Command cancelled.\n");
		return;
	}

	/* OK, it looks like this guy is serious - get a move on */
		/* save what we are moving */
	tapesave.tape_rec = tapes[tapeside].ts_tape[oldloc].tape_rec;
	tapesave.tape_cut = tapes[tapeside].ts_tape[oldloc].tape_cut;
	tapesave.tape_sec = tapes[tapeside].ts_tape[oldloc].tape_sec;
	tapesave.t_cname  = tapes[tapeside].ts_tape[oldloc].t_cname;
	tapesave.t_cart   = tapes[tapeside].ts_tape[oldloc].t_cart;

		/*now shift hole from oldloc to newloc */
	if (oldloc < newloc)
		for (i=oldloc; i <=newloc; i++)
			move_tcut(tapeside,i,i+1);
	else
		for (i=oldloc-1; i >= newloc; i--)
			move_tcut(tapeside,i+1,i);

		/* finally, move the saved entry to the new location */
	tapes[tapeside].ts_tape[newloc].tape_rec = tapesave.tape_rec;
	tapes[tapeside].ts_tape[newloc].tape_cut = tapesave.tape_cut;
	tapes[tapeside].ts_tape[newloc].tape_sec = tapesave.tape_sec;
	tapes[tapeside].ts_tape[newloc].t_cname  = tapesave.t_cname;
	tapes[tapeside].ts_tape[newloc].t_cart   = tapesave.t_cart;

	printf("Cut moved from position %d to position %d\n", oldloc+1, newloc+1);
}

move_tcut(side,dest,src)
int		side;		/* side of tape */
int		dest;		/* target destination */
int		src;		/* source location */
{		/* move tapes[side].ts_tape[src] to tape[side].ts_tape[dest] */
	tapes[side].ts_tape[dest].tape_rec = tapes[side].ts_tape[src].tape_rec;
	tapes[side].ts_tape[dest].tape_cut = tapes[side].ts_tape[src].tape_cut;
	tapes[side].ts_tape[dest].tape_sec = tapes[side].ts_tape[src].tape_sec;
	tapes[side].ts_tape[dest].t_cname  = tapes[side].ts_tape[src].t_cname;
	tapes[side].ts_tape[dest].t_cart   = tapes[side].ts_tape[src].t_cart;
}

write_tside(side,fp)
int		side;				/* side of tape to write */
FILE	*fp;				/* where to write it */
{
	int		j;				/* who says I always need an i ? */

	printf("Tape Side %d:\n", side+1);
	for (j=0; j < tapes[side].ts_num; j++)
	{		/*loop thru cuts */
		fprintf(fp,"%3d. %s %s\n\t%s\n",
			j+1,										/* cut number */
			str_time(tapes[side].ts_tape[j].tape_sec),	/* time of cut */
			tapes[side].ts_tape[j].t_cname,				/* cut name */
			tapes[side].ts_tape[j].t_cart);				/* artist name */
	}
}

zap_tape(ask)
int	ask;
{
	int		i, j;
	char	buf[BUFSIZ];
	char	*p;

	if (ask)
	{
		j = 0;		/* first time switch */
		for (i=0; i < MAXSIDES; i++)
		{
			if (j == 0)
			ontape(i);
			j++;
		}
		printf("\n***WARNING****!\n%d tape sides will be cleared!\n", j);

		printf("\nContinue? (y or n): ");
		gets(buf);
		p = clean_up(buf);
		if (*p != 'y' && *p != 'Y')
		{
			printf("Command cancelled.\n");
			return;
		}
	}

		/* zap tape sides */
	for (i=0; i < MAXSIDES; i++)
	{
		if (tapes[i].ts_num < 1) continue;	/* empty */
		for (j=0; j < tapes[i].ts_num; j++)
		{   	/* free up tape cut entry */
			free_up(tapes[i].ts_tape[j].t_cname);
			free_up(tapes[i].ts_tape[j].t_cart);
		}
		free_up(tapes[i].ts_name); tapes[i].ts_name = NULL;
		free_up(tapes[i].ts_art);  tapes[i].ts_art  = NULL;
		zap_tape_side(i);		/* clear fields */
	}
}


edit_tape()
{
	printf("Edit tape not implemented.\n");
}

fit_tape()
{		/* find all cuts that are less than some time */
	char	buf[BUFSIZ];			/* input buffer */
	int		maxtime;				/* max. time */
	int		icut;					/* loop index thru album cuts */
	int		jside;					/* loop index thru tape sides */
	int		kcut;					/* loop index thru tape cuts */
	int		secs;					/* number seconds for a song */
	int		dupl;					/* flag if duplicate song */
	int		cnt=0;					/* how many cuts have been displayed */

	tape_used(stdout,tapeside);		/* show how much is left */
	printf("Enter max. time to search for (MM:SS): ");
	gets(buf);
	maxtime = parse_time(buf);		/* convert max time to secs */
	printf("Searching for cuts less than %s (%d seconds)\n", buf, maxtime);

	for (icut=0; icut < tcuts; icut++)
	{		/* loop thru album cuts */
		secs = cuts[icut].cut_min * 60 + cuts[icut].cut_sec;
		if (secs <= maxtime)
		{		/* candidate for a fit - see if already on tape */
			dupl = 0;
			for (jside=0; ! dupl && jside < MAXSIDES; jside++)
			{		/* loop thru sides of tape */
				for (kcut=0; ! dupl && kcut < tapes[jside].ts_num; kcut++)
				{		/* make sure this is not the cut we are looking at */
					if (tapes[jside].ts_tape[kcut].tape_rec
						== cuts[icut].cut_nrec
					&&	tapes[jside].ts_tape[kcut].tape_cut
						== cuts[icut].cut_num
					) dupl++;	/* song is already on tape */
				} /* end loop thru cuts on tape */
			} /* end loop thru sides of tape */

			if (!dupl)
			{		/* not a duplicate */
				if (cnt == 0) printf("Alb Cut  Time - Song (Artist)\n");
				printf("%3d %3d %s - %s (by %s)\n",
					cuts[icut].cut_nrec+1, cuts[icut].cut_num+1,
					str_time(secs), cuts[icut].cut_name,
					cuts[icut].cut_art);
				if ((++cnt % 20) == 0)
				{		/* pause every 20 songs */
					printf("Press RETURN to continue: ");
					gets(buf);
				}
		} /* end loop looking for duplicate */
		}
	} /* end loop thru cuts on albums */

	if (cnt == 0) printf("No songs found within time limit\n");
	else
	{
		printf("Press RETURN to terminate: ");
		gets(buf);
	}
}
